@@ -132,6 +132,7 @@ class ChannelOwner(AsyncIOEventEmitter):
         self._channel: Channel = Channel(self._connection, self)
         self._initializer = initializer
         self._was_collected = False
+        self._is_internal_type = False
 
         self._connection._objects[guid] = self
         if self._parent:
@@ -156,6 +157,9 @@ class ChannelOwner(AsyncIOEventEmitter):
         self._objects[child._guid] = child
         child._parent = self
 
+    def mark_as_internal_type(self) -> None:
+        self._is_internal_type = True
+
     def _set_event_to_subscription_mapping(self, mapping: Dict[str, str]) -> None:
         self._event_to_subscription_mapping = mapping
 
@@ -197,9 +201,9 @@ class ProtocolCallback:
         if current_task:
             current_task.add_done_callback(cb)
             self.future.add_done_callback(
-                lambda _: current_task.remove_done_callback(cb)
-                if current_task
-                else None
+                lambda _: (
+                    current_task.remove_done_callback(cb) if current_task else None
+                )
             )
 
 
@@ -243,9 +247,9 @@ class Connection(EventEmitter):
         self._error: Optional[BaseException] = None
         self.is_remote = False
         self._init_task: Optional[asyncio.Task] = None
-        self._api_zone: contextvars.ContextVar[
-            Optional[ParsedStackTrace]
-        ] = contextvars.ContextVar("ApiZone", default=None)
+        self._api_zone: contextvars.ContextVar[Optional[ParsedStackTrace]] = (
+            contextvars.ContextVar("ApiZone", default=None)
+        )
         self._local_utils: Optional["LocalUtils"] = local_utils
         self._tracing_count = 0
         self._closed_error: Optional[Exception] = None
@@ -294,6 +298,8 @@ class Connection(EventEmitter):
             # To prevent 'Future exception was never retrieved' we ignore all callbacks that are no_reply.
             if callback.no_reply:
                 continue
+            if callback.future.cancelled():
+                continue
             callback.future.set_exception(self._closed_error)
         self._callbacks.clear()
         self.emit("close")
@@ -353,7 +359,7 @@ class Connection(EventEmitter):
             "params": self._replace_channels_with_guids(params),
             "metadata": metadata,
         }
-        if self._tracing_count > 0 and frames and object._guid != "localUtils":
+        if self._tracing_count > 0 and frames and not object._is_internal_type:
             self.local_utils.add_stack_to_tracing_no_reply(id, frames)
 
         self._transport.send(message)
